@@ -890,7 +890,7 @@ static const FieldMap stretch_map[] = {
 static const FieldMap gravity_map[] = {
   { PANGO_GRAVITY_SOUTH, "Not-Rotated" },
   { PANGO_GRAVITY_SOUTH, "South" },
-  { PANGO_GRAVITY_SOUTH, "Upside-Down" },
+  { PANGO_GRAVITY_NORTH, "Upside-Down" },
   { PANGO_GRAVITY_NORTH, "North" },
   { PANGO_GRAVITY_EAST,  "Rotated-Left" },
   { PANGO_GRAVITY_EAST,  "East" },
